%%% This function has two actions                                        %%%
%%%     1. Initialize the three UDP sockects (DATEX, PUMP 1 and Pump 2)  %%%
%%%     2. Start the timer 1, timer 2, timer 3, timer 4 (sampling time)  %%%
%%% Luis Alberto Paz Surez                                              %%%
%%% 2010-08-17                                                           %%%

function Function_Start()
global Timer_1 Timer_Pump_1 Timer_Pump_2 Timer_Pump_3 Timer_Simulator;
global sockcon_1 sockcon_2 sockcon_3 sockcon_4;
global Enabled_DATEX Enabled_Syringe_1 Enabled_Syringe_2  Enabled_Syringe_3;
global RB_Real_Time RB_Simulator Button_Start Button_Stop;
global DATEX_UDP_Port_Read;
global Pump_1_UDP_Port_Read;
global Pump_2_UDP_Port_Read;
global Pump_3_UDP_Port_Read;
global Time_Sim Ts TS_Plot Index;
global NMB_Model NMB_Model_Filt Ref_NMB Control_1 SP_NMB E_NMB Temp Theta
global Kc Ti Td Flag_10 Flag_Change Flag_Start_Control Flag_T50 Flag_PCA Flag_Filter B_Filt;
global Flag_Tunning Master;
global ALPHA GAMMA C50 ;
global X_NMB  A_Dcrt B_Dcrt C_Dcrt 

set(Enabled_DATEX,'Enable','off')
set(Enabled_Syringe_1,'Enable','off')
set(Enabled_Syringe_2,'Enable','off')
set(Enabled_Syringe_3,'Enable','off')
set(RB_Real_Time,'Enable','off')
set(RB_Simulator,'Enable','off')
set(Button_Start,'Enable','off')
set(Button_Stop,'Enable','on')
pnet('closeall');

if (get(RB_Real_Time,'Value') == get(RB_Real_Time,'Max'))
    %%%%%%%%%%% Work in Real Mode   %%%%%%%%%%%%%%%%%%%
    Status = get(Enabled_DATEX,'value');
    if Status == 1
        % Initialize the UDP sockets
        sockcon_1 = pnet('udpsocket', DATEX_UDP_Port_Read);
        pnet(sockcon_1,'setwritetimeout',1);
        pnet(sockcon_1,'setreadtimeout',0);
        % start the timer 1 (DATEX)
        out = get(Timer_1, 'Running');
        flag = strcmp(out,'off');
        if flag == 1
            start(Timer_1);
        end
    end
    Status = get(Enabled_Syringe_1,'value');
    if Status == 1
        % Initialize the UDP sockets
        sockcon_2 = pnet('udpsocket', Pump_1_UDP_Port_Read);
        pnet(sockcon_2,'setwritetimeout',1);
        pnet(sockcon_2,'setreadtimeout',0);
        % start the timer 2 (Pump 1)
        out = get(Timer_Pump_1, 'Running');
        flag = strcmp(out,'off');
        if flag == 1
            start(Timer_Pump_1);
        end
    end
    
    Status = get(Enabled_Syringe_2,'value');
    if Status == 1
        % Initialize the UDP sockets
        sockcon_3 = pnet('udpsocket', Pump_2_UDP_Port_Read);
        pnet(sockcon_3,'setwritetimeout',1);
        pnet(sockcon_3,'setreadtimeout',0);
        % start the timer 3 (Pump 2)
        out = get(Timer_Pump_2, 'Running');
        flag = strcmp(out,'off');
        if flag == 1
            start(Timer_Pump_2);
        end
    end
    
    Status = get(Enabled_Syringe_3,'value');
    if Status == 1
        % Initialize the UDP sockets
        sockcon_4 = pnet('udpsocket', Pump_3_UDP_Port_Read);
        pnet(sockcon_4,'setwritetimeout',1);
        pnet(sockcon_4,'setreadtimeout',0);
        % start the timer 4 (Pump 3)
        out = get(Timer_Pump_3, 'Running');
        flag = strcmp(out,'off');
        if flag == 1
            start(Timer_Pump_3);
            display('star Timer-3')
        end
    end
else
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%% Work in Simulation Mode   %%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    display('work in simulation mode')
    Model = get(findobj('tag','PpM_Model'),'Value');
    Controller = get(findobj('tag','PpM_Control'),'Value');
    % Inicializao
    M = get(findobj('tag','PpM_Case_Model'),'Value');        % Modelo a utilizar (Model)
    Mc = get(findobj('tag','PpM_Case_Control'),'Value');     % Modelo a utilizar (Controller)
    Ts=1/3;             % Periodo de amostragem (minutos)
    Ref_NMB = str2double(get(findobj('tag','BEdit_SP_NMB'),'string'));    % Referencia
    Ubolus=500*1/Ts;    % Bolus Inicial
    TS_Plot = 0;
    
    switch Model
        case 1
            % Reduced Model
            load THETA_database_PEM_final
            ALPHA=THETA(1,M);
            GAMMA=THETA(2,M);
            k0=5; k1=1; k2=4; k3=10;
            C50=k0*0.6487;
            A=[-(k1+k2+k3)*ALPHA -(k1*k2+k1*k3+k2*k3)*ALPHA^2 -k1*k2*k3*ALPHA^3; 1 0 0; 0 1 0];
            B=[1 0 0]'; 
            C=[0 0 k1*k2*k3*ALPHA^3];
            [A_Dcrt, B_Dcrt]= c2d(A,B,Ts);
            C_Dcrt = C;            
            X_NMB = [0 0 0]';
            
        case 2
            % PK/PD Model
            load Param_NMB      % Parametos dos 100 modelos (C50_v Sv a1 a2 lambda1 lambda1 lambda_v tau_v)
            Theta=[a1(M) lambda1(M) a2(M) lambda2(M) lambda_v(M) tau_v(M) C50_v(M) S_v(M)];
            A=[-Theta(2) 0 0 0;0 -Theta(4) 0 0; Theta(5) Theta(5) -Theta(5) 0; 0 0 1/Theta(6) -1/Theta(6)];
            B=[Theta(1) Theta(3) 0 0]';
            C=[0 0 0 1];
            [A_Dcrt, B_Dcrt]=c2d(A,B,Ts);   % Espao de estado discreto
            C_Dcrt = C;                     % Espao de estado discreto
            X_NMB=[0 0 0 0]';
                       
        case 3
            % Data File
            
    end
    
    %Init the controlles parameters
    load Param_NMB      % Parametos dos 100 modelos (C50_v Sv a1 a2 lambda1 lambda1 lambda_v tau_v)
    switch Controller
        case 1
            % Proportional Controller Model
            R = 0.0037; L = 2.6114; S_c = S_v(Mc); C50_c = C50_v(Mc);
            RD = (-Ref_NMB^2*S_c*(100/Ref_NMB-1)^(1-1/S_c))/(100*C50_c);
            Kc = 1/(R*L*RD);     % Constante proporcional
            Ti = Inf;            % Tempo de Integrao (Diferencia com Magalhaes Ti = 2*L)
            Td = 0;              % Tempo Derivativo
        case 2
            % PID controllers with fixed parameters
            R = 0.0037; L = 2.6114; S_c = S_v(Mc); C50_c = C50_v(Mc);
            RD = (-Ref_NMB^2*S_c*(100/Ref_NMB-1)^(1-1/S_c))/(100*C50_c);
            Kc = 1/(R*L*RD);     % Constante proporcional
            Ti = 2*L;            % Tempo de Integrao (Diferencia com Magalhaes Ti = 2*L)
            Td = L/2;            % Tempo Derivativo
            
        case 3
            % PID controllers with auto.tunning (T50 Method)
            % The PID parameters are determined when the NMB reaches 50%  
            % (in Timer_Simulator_Function()) 
                        
        case 4
            % PID controllers with auto.tunning (PCA Method)
            % The PID parameters are determined after reading 30 values of
            % NMB (in Timer_Simulator_Function())
           
    end 
    
    % Condie iniciais
    Index = 1;
    NMB_Model = zeros(1);
    NMB_Model_Filt = zeros(1);
    Control_1 = zeros(1);
    SP_NMB = zeros(1);
    E_NMB = zeros(1);
    Temp = zeros(1);
    Time_Sim = zeros(1);
    Flag_10 = 0;
    Flag_Change = 0;
    Flag_Start_Control = 0;
    Flag_T50 = 0;
    Flag_PCA = 0;
    Flag_Tunning = 0;
    B_Filt = fir1(10,0.0001);
    
    NMB_Model(1) = 100;
    NMB_Model_Filt(1) = 100;
    Time_Sim(1) = 0;
    Control_1(1) = Ubolus;
    SP_NMB(1)= Ref_NMB;
    E_NMB(1) = NMB_Model(1) - SP_NMB(1);
    Temp_0 = 0; %Kc*Ts*E(1)/Ti;
    Temp(1) = Temp_0;
 
    Flag_Filter = get(findobj('tag','Ch_Filter'),'value');
    Master = get(findobj('tag','RB_Auto'),'value');
    start(Timer_Simulator)
end
end


